@@ -332,7 +332,7 @@ view with lots of 1-track elements."
   :type 'function)
 
 (defcustom emms-browser-covers
-  '("cover_small.jpg" "cover_med.jpg" "cover_large.jpg")
+  '("cover_small" "cover_med" "cover_large")
   "*Control how cover images are found.
 Can be either a list of small, medium and large images (large
 currently not used), a function which takes a directory and one
@@ -341,6 +341,17 @@ path to the cover, or nil to turn off cover loading."
   :group 'emms-browser
   :type '(choice list function boolean))
 
+(defcustom emms-browser-covers-file-extensions
+  '("jpg" "jpeg" "png" "gif" "bmp")
+  "*File extensions accepted for `emms-browser-covers'.
+Should be a list of extensions as strings.  Should be set before
+emms-browser is required."
+  :group 'emms-browser
+  :type '(repeat (string :tag "Extension")))
+
+(defconst emms-browser--covers-filename nil
+  "*List of potential cover art names.")
+
 (defcustom emms-browser-default-covers nil
   "*A list of default images to use if a cover isn't found."
   :group 'emms-browser
@@ -363,7 +374,7 @@ Ues nil for no sorting."
   :type 'function)
 
 (defcustom emms-browser-alpha-sort-function
-  'string<
+  (if (functionp 'string-collate-lessp) 'string-collate-lessp 'string<)
   "*How to sort artists/albums/etc. in the browser.
 Use nil for no sorting."
   :group 'emms-browser
@@ -473,6 +484,8 @@ Called once for each directory."
     (define-key map (kbd "W a p") 'emms-browser-lookup-album-on-pitchfork)
     (define-key map (kbd ">") 'emms-browser-next-filter)
     (define-key map (kbd "<") 'emms-browser-previous-filter)
+    (define-key map (kbd "+") 'emms-volume-raise)
+    (define-key map (kbd "-") 'emms-volume-lower)
     map)
   "Keymap for `emms-browser-mode'.")
 
@@ -671,7 +684,16 @@ compilations, etc."
            db)
   (emms-with-inhibit-read-only-t
    (let ((sort-fold-case t))
-     (sort-lines nil (point-min) (point-max)))))
+     (if emms-browser-alpha-sort-function
+         (progn
+           (goto-char (point-min))
+           (sort-subr nil
+                      #'forward-line #'end-of-line
+                      (lambda () (buffer-substring-no-properties
+                             (line-beginning-position) (line-end-position)))
+                      nil
+                      emms-browser-alpha-sort-function))
+       (sort-lines nil (point-min) (point-max))))))
 
 (defun case-fold-string= (a b)
   (eq t (compare-strings a nil nil b nil nil t)))
@@ -1250,7 +1272,7 @@ Return the previous point-max before adding."
                 (forward-line (or direction 1)))
       (unless (eq (emms-browser-bdata-type
                    (emms-browser-bdata-at-point)) 'info-title)
-        (setq continue)))))
+        (setq continue nil)))))
 
 (defun emms-browser-prev-non-track ()
   "Jump to the previous non-track element."
@@ -1291,12 +1313,14 @@ Return the previous point-max before adding."
   "Do we need to seed (random)?")
 
 (defun emms-browser-goto-random ()
+  "Move cursor to random item with the lowest visible level."
   (interactive)
   (when emms-browser-seed-pending
     (random t)
     (setq emms-browser-seed-pending nil))
-  (goto-char (point-min))
-  (forward-line (1- (random (count-lines (point-min) (point-max))))))
+  (while (progn (goto-char (point-min))
+                (forward-line (1- (random (count-lines (point-min) (point-max)))))
+                (emms-browser-subitems-visible))))
 
 (defun emms-browser-view-in-dired (&optional bdata)
   "View the current directory in dired."
@@ -1599,6 +1623,19 @@ included."
 ;; Album covers
 ;; --------------------------------------------------
 
+(defun emms-browser--build-cover-filename ()
+  "Build `emms-browser--covers-filename'.
+
+Based on from `emms-browser-covers' and
+`emms-browser-covers-file-extensions'."
+  (setq emms-browser--covers-filename
+        (mapcar (lambda (cover)
+                  (if (file-name-extension cover)
+                      (list cover)
+                    (mapcar (lambda (ext) (concat cover "." ext))
+                            emms-browser-covers-file-extensions)))
+                emms-browser-covers)))
+
 (defun emms-browser-get-cover-from-album (bdata &optional size)
   (assert (eq (emms-browser-bdata-type bdata) 'info-album))
   (let* ((track1data (emms-browser-bdata-data bdata))
@@ -1608,6 +1645,8 @@ included."
 
 (defun emms-browser-get-cover-from-path (path &optional size)
   "Return a cover filename, if it exists."
+  (unless emms-browser--covers-filename
+    (emms-browser--build-cover-filename))
   (unless size
     (setq size 'medium))
   (let* ((size-idx (cond
@@ -1620,10 +1659,13 @@ included."
             (funcall emms-browser-covers (file-name-directory path) size))
            ((and (listp emms-browser-covers)
                  (nth size-idx emms-browser-covers))
-            (concat (file-name-directory path)
-                    (nth size-idx emms-browser-covers))))))
-    (if (and cover
-             (file-readable-p cover))
+            (car (delq nil
+                       (mapcar (lambda (cover)
+                                 (let ((coverpath
+                                        (concat (file-name-directory path) cover)))
+                                   (and (file-exists-p coverpath) coverpath)))
+                               (nth size-idx emms-browser--covers-filename))))))))
+    (if (and cover (file-readable-p cover))
         cover
       ;; no cover found, use default
       (when emms-browser-default-covers
